/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core;

import org.sonar.plugins.api.EditableProperties;
import org.sonar.plugins.api.EditableProperty;
import org.sonar.plugins.api.Extension;
import org.sonar.plugins.api.Plugin;
import org.sonar.plugins.core.charts.XradarChart;
import org.sonar.plugins.core.codecoveragesourceviewer.CodeCoverageSourceViewer;
import org.sonar.plugins.core.complexity.ComplexityPercentDistributionJob;
import org.sonar.plugins.core.complexity.PackageComplexityCountDistributionJob;
import org.sonar.plugins.core.complexity.SumComplexityCountDistributionJob;
import org.sonar.plugins.core.defaultsourceviewer.DefaultSourceViewer;
import org.sonar.plugins.core.metrics.UserManagedMetrics;
import org.sonar.plugins.core.resourceviewer.ResourceViewer;
import org.sonar.plugins.core.rules.RespectForMandatoryRulesJob;
import org.sonar.plugins.core.rules.RespectForOptionalRulesJob;
import org.sonar.plugins.core.rules.RespectForRulesJob;
import org.sonar.plugins.core.rules.ViolationsJob;
import org.sonar.plugins.core.sources.ImportSourcesMavenCollector;
import org.sonar.plugins.core.unittests.CodeCoverageMavenCollectorFilter;
import org.sonar.plugins.core.unittests.ProjectCodeCoverageJob;
import org.sonar.plugins.core.unittests.UnitTestsConsolidationJob;
import org.sonar.plugins.core.violationssourceviewer.ViolationsSourceViewer;
import org.sonar.plugins.core.clouds.Clouds;
import org.sonar.plugins.core.events.SetAutomaticEventsJob;

import java.util.ArrayList;
import java.util.List;

@EditableProperties({
  @EditableProperty(key = CodeCoverageMavenCollectorFilter.PROP_KEY_CODECOVERAGE_PLUGINS, defaultValue = CodeCoverageMavenCollectorFilter.PROP_DEFAULT_CODECOVERAGE_PLUGINS, name = "Code coverage plugin", description = "Key of the code coverage plugin to use, e.g. cobertura. It can be explicitly set on projects, just add the parameter <code>" + CodeCoverageMavenCollectorFilter.PROP_KEY_CODECOVERAGE_PLUGINS + "</code> to maven (-D parameter or pom.xml property). <br/>" +
    "Default is 'clover,cobertura', so Clover is used if a valid license is found, else Cobertura."),
  @EditableProperty(key = CorePlugin.PROP_KEY_IMPORT_SOURCES, defaultValue = "" + CorePlugin.PROP_DEFAULT_IMPORT_SOURCES, name = "Import sources", description = "Set to false if sources should not be displayed, e.g. for security reasons. It can also be set explicitly on projects, just add the parameter <code>" + CorePlugin.PROP_KEY_IMPORT_SOURCES + "=false</code> to maven (-D parameter or pom.xml property)."),
  @EditableProperty(key = CorePlugin.PROP_KEY_CRC_FACTOR, defaultValue = "" + CorePlugin.PROP_DEFAULT_CRC_FACTOR, name = "Acceleration factor for Coding Rules Compliance", description = CorePlugin.PROP_DEFAULT_CRC_DESC),
  @EditableProperty(key = CorePlugin.PROP_KEY_TENDENCY_DEPTH, defaultValue = "" + CorePlugin.PROP_DEFAULT_TENDENCY_DEPTH, name = "Tendency depth", description = CorePlugin.PROP_DEFAULT_TENDENCY_DESC)
    })
public class CorePlugin implements Plugin {
  public static final String PLUGIN_KEY = "core";

  public static final String PROP_KEY_IMPORT_SOURCES = "sonar.importSources";
  public static final boolean PROP_DEFAULT_IMPORT_SOURCES = true;

  public static final String PROP_KEY_TENDENCY_DEPTH = "tendency.depth";
  public static final int PROP_DEFAULT_TENDENCY_DEPTH = 3;
  public static final String PROP_DEFAULT_TENDENCY_DESC ="This parameter is related to the tendencies, the little arrows showing the evolution of measures.<br/>In order to calculate the tendencies, Sonar takes the last [tendency depth] measures, determines an estimated slope and displays it using the arrows.<br/>";

  public static final String PROP_KEY_CRC_FACTOR = "sonar.core.crcFactor";
  public static final int PROP_DEFAULT_CRC_FACTOR = 3;
  public static final String PROP_DEFAULT_CRC_DESC ="Coding Rules Compliance is : 1 - (number of violations * acceleration factor) / lines of code.<br/>"+
      "When increasing the acceleration factor, the Coding Rules Compliance(CRC) decreases quicker at each violation. "+
      "For example, on a project with 10'000 lines of codes and 200 violations:<br/>"+
      "- with an acceleration factor of 3, CRC is 94%<br/>"+
      "- with an acceleration factor of 5, CRC is 90%<br/>"+
      "This can be used when you have reached such a maturity level that changes are not impacting CRC anymore.";

  public String getKey() {
    return PLUGIN_KEY;
  }

  public String getName() {
    return "General";
  }

  public String getDescription() {
    return "";
  }

  public List<Class<? extends Extension>> getExtensions() {
    List<Class<? extends Extension>> extensions = new ArrayList<Class<? extends Extension>>();

    // metrics
    extensions.add(UserManagedMetrics.class);

    // resource viewer
    extensions.add(ResourceViewer.class);

    // source viewers
    extensions.add(DefaultSourceViewer.class);
    extensions.add(CodeCoverageSourceViewer.class);
    extensions.add(ViolationsSourceViewer.class);

    // web pages
    //extensions.add(TestsPage.class);
    extensions.add(Clouds.class);

    // maven
    extensions.add(CodeCoverageMavenCollectorFilter.class);
    extensions.add(ImportSourcesMavenCollector.class);

    // jobs
    extensions.add(ProjectCodeCoverageJob.class);
    extensions.add(UnitTestsConsolidationJob.class);
    extensions.add(RespectForMandatoryRulesJob.class);
    extensions.add(RespectForOptionalRulesJob.class);
    extensions.add(RespectForRulesJob.class);
    extensions.add(ViolationsJob.class);
    extensions.add(ComplexityPercentDistributionJob.class);
    extensions.add(PackageComplexityCountDistributionJob.class);
    extensions.add(SumComplexityCountDistributionJob.class);
    extensions.add(SetAutomaticEventsJob.class);

    // charts
    extensions.add(XradarChart.class);

    return extensions;
  }

  public String toString() {
    return getKey();
  }
}
