/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.clouds.client.widget;

import com.google.gwt.user.client.ui.*;
import org.sonar.plugins.api.web.gwt.client.SonarWidget;
import org.sonar.plugins.api.web.gwt.client.rest.model.Measure;
import org.sonar.plugins.api.web.gwt.client.rest.model.Project;
import org.sonar.plugins.api.web.gwt.client.rest.model.Snapshot;
import org.sonar.plugins.core.clouds.client.Calculator;
import org.sonar.plugins.core.clouds.client.model.CloudElement;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;
import java.util.Map;

public class ClassCloudsWidget extends SonarWidget {

  private Panel main = new FlowPanel();
  private String sizeMetric;
  private String colorMetric;
  private TabWidget tabClouds;

  private float minSizePercent = 60f;
  private float maxSizePercent = 240f;

  private Calculator sizeCalculator = new Calculator(minSizePercent, maxSizePercent);
  private Calculator colorCalculator = new Calculator(0f, 100f);

  public ClassCloudsWidget(List<Project> projects, String sizeMetric, String colorMetric, final TabWidget tabClouds) {
    this.sizeMetric = sizeMetric;
    this.colorMetric = colorMetric;
    this.tabClouds = tabClouds;
    initWidget(main);

    List<CloudElement> cloudElements = getCloudElements(projects);
    createClouds(cloudElements);
  }

  private List<CloudElement> getCloudElements(List<Project> projects) {
    List<CloudElement> tagList = new ArrayList<CloudElement>();
    for (Project project : projects) {
      Measure sizeMeasure = getMeasure(project, sizeMetric);
      Measure colorMeasure = getMeasure(project, colorMetric);

      if (sizeMeasure != null && colorMeasure != null) {
        Integer size = getMeasureValue(sizeMeasure.getValue());
        float color = Float.valueOf(colorMeasure.getValue());

        tagList.add(new CloudElement(project, size, color));
        sizeCalculator.updateMaxAndMin(Float.valueOf(size.toString()));
      }
    }
    Collections.sort(tagList);
    return tagList;
  }

  private Integer getMeasureValue(String textValue) {
    Float floatValue = (Float.valueOf(textValue) * 100.0f);
    return floatValue.intValue();
  }

  private Measure getMeasure(Project project, String metricToFound) {
    List<Snapshot> snapshots = project.getSnapshots();
    if (snapshots != null && !snapshots.isEmpty()) {
      Map<String, Measure> measures = snapshots.get(0).getMeasures();
      if (measures != null && !measures.isEmpty()) {
        for (Map.Entry<String, Measure> entry : measures.entrySet()) {
          if (entry.getKey().equals(metricToFound)) {
            return entry.getValue();
          }
        }
      }
    }
    return null;
  }

  private void createClouds(List<CloudElement> cloudElements) {
    for (CloudElement tag : cloudElements) {
      HTML className = new HTML(
        "<span style=\"font-size:" + Integer.toString(sizeCalculator.getFontSizePercent(tag.getFontSize())) +
          "%; color:" + colorCalculator.getFontColor(tag.getFontColor()) + "\" >" +
          tag.getProject().getName() + "</span>\n");
      className.setStyleName("inline");

      Hyperlink link = createLink(tag);
      link.setHTML(className.getHTML());
      main.add(link);
    }
  }

  private Hyperlink createLink(final CloudElement tag) {
    Hyperlink link = new Hyperlink();
    link.setStyleName("tag inline");
    String tooltip = getTooltip(tag.getProject());
    link.getElement().setAttribute("title", tooltip);
    link.getElement().setAttribute("rel", tooltip);

    String sizeCss = Float.toString(maxSizePercent / 100f) + "em";
    link.setHeight(sizeCss);
    link.addClickListener(new ClickListener() {
      public void onClick(Widget widget) {
        tabClouds.refreshSourceTab(tag);
      }
    });

    return link;
  }

  private String getTooltip(Project project) {
    Measure sizeMeasure = getMeasure(project, sizeMetric);
    String sizeMetricName = sizeMeasure.getMetricName();
    String sizeMetricValue = sizeMeasure.getFormattedValue();

    Measure colorMeasure = getMeasure(project, colorMetric);
    String colorMetricName = colorMeasure.getMetricName();
    String colorMetricValue = colorMeasure.getFormattedValue();

    return sizeMetricName + " : " + sizeMetricValue + ", " + colorMetricName + " : " + colorMetricValue;
  }


}
