/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.rules;

import ch.hortis.sonar.model.*;
import ch.hortis.sonar.service.MeasureKey;
import org.apache.commons.collections.Bag;
import org.apache.commons.collections.bag.HashBag;
import org.apache.commons.lang.ObjectUtils;
import org.sonar.commons.DaoFacade;
import org.sonar.commons.measures.Measure;
import org.sonar.plugins.api.Language;
import org.sonar.plugins.api.Languages;
import org.sonar.plugins.api.jobs.AbstractJob;
import org.sonar.plugins.api.jobs.Resource;
import org.sonar.plugins.api.metrics.CoreMetrics;

import java.util.Arrays;
import java.util.List;

public class ViolationsJob extends AbstractJob {
  private Bag rulesBag = new HashBag();
  private Bag categoriesBag = new HashBag();
  private List<RulesCategory> categories;
  private DaoFacade dao;
  private RulesProfile profile;

  @Override
  public List<Metric> generatesMetrics() {
    return Arrays.asList(CoreMetrics.RULES_MANDATORY_VIOLATIONS_COUNT, CoreMetrics.RULES_OPTIONAL_VIOLATIONS_COUNT, CoreMetrics.RULES_VIOLATIONS_COUNT);
  }

  public ViolationsJob(DaoFacade dao) {
    this.dao = dao;
  }

  protected Language shouldExecuteOnLanguage() {
    return null; // all languages
  }

  public boolean shouldExecuteOnResource(Resource node) {
    return !node.isJavaTest();
  }

  public boolean shouldExecuteOnProject(Resource project) {
    categories = dao.getRulesDao().getCategories();
    Language language = Languages.getLanguageByProjectQualifier(project.getQualifier());
    profile = dao.getRulesDao().getActiveProfile(language.getKey());
    return super.shouldExecuteOnProject(project);
  }

  public void execute(Resource resource) {
    if (resource.isClass()) {
      count(resource, RuleFailureLevel.ERROR, CoreMetrics.RULES_MANDATORY_VIOLATIONS_COUNT);
      count(resource, RuleFailureLevel.WARNING, CoreMetrics.RULES_OPTIONAL_VIOLATIONS_COUNT);
      count(resource, null, CoreMetrics.RULES_VIOLATIONS_COUNT);

    } else {
      consolidate(resource, CoreMetrics.RULES_MANDATORY_VIOLATIONS_COUNT);
      consolidate(resource, CoreMetrics.RULES_OPTIONAL_VIOLATIONS_COUNT);
      consolidate(resource, CoreMetrics.RULES_VIOLATIONS_COUNT);
    }
  }

  private void count(Resource node, RuleFailureLevel level, Metric metric) {
    rulesBag.clear();
    categoriesBag.clear();
    int total = 0;

    for (RuleFailure ruleFailure : node.getRuleFailures()) {
      if (level == null || level.equals(ruleFailure.getLevel())) {
        total++;
        rulesBag.add(ruleFailure.getRule());
        categoriesBag.add(ruleFailure.getRule().getRulesCategory().getId());
      }
    }

    node.addMeasure(new MeasureKey(metric, null, null), new Double(total));

    for (RulesCategory category : categories) {
      int count = categoriesBag.getCount(category.getId());
      node.addMeasure(new MeasureKey(metric, category.getId(), null), (double) count);
    }

    for (ActiveRule activeRule : profile.getActiveRules()) {
      Rule rule = activeRule.getRule();
      int count = rulesBag.getCount(rule);
      node.addMeasure(new MeasureKey(metric, rule.getRulesCategory().getId(), rule), (double) count);
    }
  }

  private void consolidate(Resource resource, Metric metric) {
    MeasureKey measureKeyRoot = new MeasureKey(metric, null, null);
    resource.addMeasure(measureKeyRoot, getChildrenSumMeasure(resource, measureKeyRoot));

    for (RulesCategory category : categories) {
      MeasureKey measureKeyCat = new MeasureKey(metric, category.getId(), null);
      resource.addMeasure(measureKeyCat, getChildrenSumMeasure(resource, measureKeyCat));
    }

    for (ActiveRule activeRule : profile.getActiveRules()) {
      MeasureKey measureKeyRule = new MeasureKey(metric, activeRule.getRule().getRulesCategory().getId(), activeRule.getRule());
      resource.addMeasure(measureKeyRule, getChildrenSumMeasure(resource, measureKeyRule));
    }
  }

  private Double getChildrenSumMeasure(Resource resource, MeasureKey measureKey) {
    Double value = 0.0;

    for (Measure measure : resource.getChildrenMeasures(measureKey)) {
      if (hasSameRuleAndCategory(measure, measureKey)) {
        value += measure.getValue();
      }
    }
    return value;
  }

  private boolean hasSameRuleAndCategory(Measure measure, MeasureKey measureKey) {
    return ObjectUtils.equals(measureKey.getRule(), measure.getRule()) &&
        ObjectUtils.equals(measureKey.getRulesCategoryId(), measure.getRulesCategoryId());
  }
}
