/*
 * Sonar, entreprise quality control tool.
 * Copyright (C) 2007-2008 Hortis-GRC SA
 * mailto:be_agile HAT hortis DOT ch
 *
 * Sonar is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 3 of the License, or (at your option) any later version.
 *
 * Sonar is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with Sonar; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02
 */
package org.sonar.plugins.core.unittests;

import org.apache.commons.configuration.Configuration;
import org.apache.commons.lang.StringUtils;
import org.apache.maven.plugin.logging.Log;
import org.sonar.plugins.api.Plugin;
import org.sonar.plugins.api.Plugins;
import org.sonar.plugins.api.maven.CollectsCodeCoverage;
import org.sonar.plugins.api.maven.CollectsUnitTests;
import org.sonar.plugins.api.maven.MavenCollector;
import org.sonar.plugins.api.maven.MavenCollectorFilter;
import org.sonar.plugins.api.maven.model.MavenPom;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

public class CodeCoverageMavenCollectorFilter implements MavenCollectorFilter {
  public static final String PROP_KEY_CODECOVERAGE_PLUGINS = "sonar.core.codeCoveragePlugin";
  public static final String PROP_DEFAULT_CODECOVERAGE_PLUGINS = "clover,cobertura";

  public static final String PROP_KEY_UNIT_TESTS_PLUGIN = "sonar.core.unitTestsPlugin";
  public static final String PROP_DEFAULT_UNIT_TESTS_PLUGIN = "surefire";

  private Plugins plugins;
  private Log log;
  private String[] codeCoveragePlugins = null;
  private String unitTestsPlugin = null;

  public CodeCoverageMavenCollectorFilter(Configuration configuration, Plugins plugins, Log log) {
    this(
        configuration.getStringArray(PROP_KEY_CODECOVERAGE_PLUGINS),
        configuration.getString(PROP_KEY_UNIT_TESTS_PLUGIN),
        plugins,
        log);
  }

  public CodeCoverageMavenCollectorFilter(String[] codeCoveragePlugins, String unitTestsPlugin, Plugins plugins, Log log) {
    if (codeCoveragePlugins == null || codeCoveragePlugins.length == 0) {
      this.codeCoveragePlugins = StringUtils.split(PROP_DEFAULT_CODECOVERAGE_PLUGINS, ",");
    } else {
      this.codeCoveragePlugins = codeCoveragePlugins;
    }
    this.unitTestsPlugin = StringUtils.defaultIfEmpty(unitTestsPlugin, PROP_DEFAULT_UNIT_TESTS_PLUGIN);
    this.plugins = plugins;
    this.log = log;
  }

  public void filter(List<MavenCollector> collectors, MavenPom pom) {
    if (collectors != null) {
      List<MavenCollector> filteredCollectors = removeCodeCoverageAndUnitTestsCollectors(collectors);

      MavenCollector codeCoverageCollector = searchCollector(codeCoveragePlugins, filteredCollectors, CollectsCodeCoverage.class, pom);
      if (codeCoverageCollector!=null) {
        collectors.add(codeCoverageCollector);
      }

      MavenCollector unitTestsCollector = searchCollector(new String[]{unitTestsPlugin}, filteredCollectors, CollectsUnitTests.class, pom);
      if (unitTestsCollector!=null) {
        collectors.add(unitTestsCollector);
      }

      makeThemMeet((CollectsCodeCoverage)codeCoverageCollector, (CollectsUnitTests)unitTestsCollector);
    }
  }

  private void makeThemMeet(CollectsCodeCoverage codeCoverageCollector, CollectsUnitTests unitTestsCollector) {
    if (codeCoverageCollector!=null && unitTestsCollector!=null) {
      codeCoverageCollector.configure(unitTestsCollector);
    }
  }

  private List<MavenCollector> removeCodeCoverageAndUnitTestsCollectors(List<MavenCollector> collectors) {
    List<MavenCollector> filteredCollectors = new ArrayList<MavenCollector>();
    for (Iterator<MavenCollector> it = collectors.iterator(); it.hasNext();) {
      MavenCollector collector = it.next();
      if (collector instanceof CollectsCodeCoverage || collector instanceof CollectsUnitTests) {
        filteredCollectors.add(collector);
        it.remove();
      }
    }
    return filteredCollectors;
  }

  private MavenCollector searchCollector(String[] pluginKeys, List<MavenCollector> filteredCollectors, Class clazzToImplement, MavenPom pom) {
    if (pluginKeys != null) {
      for (String pluginKey : pluginKeys) {
        for (MavenCollector collector : filteredCollectors) {
          Plugin plugin = plugins.getPluginByExtension(collector);
          if (clazzToImplement.isAssignableFrom(collector.getClass()) && pluginKey.equals(plugin.getKey()) && collector.shouldCollectOn(pom)) {
            return collector;
          }
        }
      }
    }
    return null;
  }
}